@@ -1,45 +1,20 @@
 ;;; srfi-1.scm --- List Library
 
-;; 	Copyright (C) 2001, 2002, 2004 Free Software Foundation, Inc.
+;; 	Copyright (C) 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
 ;;
-;; This program is free software; you can redistribute it and/or
-;; modify it under the terms of the GNU General Public License as
-;; published by the Free Software Foundation; either version 2, or
-;; (at your option) any later version.
-;;
-;; This program is distributed in the hope that it will be useful,
+;; This library is free software; you can redistribute it and/or
+;; modify it under the terms of the GNU Lesser General Public
+;; License as published by the Free Software Foundation; either
+;; version 2.1 of the License, or (at your option) any later version.
+;; 
+;; This library is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
 ;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;; General Public License for more details.
-;;
-;; You should have received a copy of the GNU General Public License
-;; along with this software; see the file COPYING.  If not, write to
-;; the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
-;; Boston, MA 02111-1307 USA
-;;
-;; As a special exception, the Free Software Foundation gives permission
-;; for additional uses of the text contained in its release of GUILE.
-;;
-;; The exception is that, if you link the GUILE library with other files
-;; to produce an executable, this does not by itself cause the
-;; resulting executable to be covered by the GNU General Public License.
-;; Your use of that executable is in no way restricted on account of
-;; linking the GUILE library code into it.
-;;
-;; This exception does not however invalidate any other reasons why
-;; the executable file might be covered by the GNU General Public License.
-;;
-;; This exception applies only to the code released by the
-;; Free Software Foundation under the name GUILE.  If you copy
-;; code from other Free Software Foundation releases into a copy of
-;; GUILE, as the General Public License permits, the exception does
-;; not apply to the code that you add in this way.  To avoid misleading
-;; anyone as to the status of such modified files, you must delete
-;; this exception notice from them.
-;;
-;; If you write modifications of your own for GUILE, it is your choice
-;; whether to permit this exception to apply to your modifications.
-;; If you do not wish that, delete this exception notice.
+;; Lesser General Public License for more details.
+;; 
+;; You should have received a copy of the GNU Lesser General Public
+;; License along with this library; if not, write to the Free Software
+;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
 ;;; Author: Martin Grabmueller <mgrabmue@cs.tu-berlin.de>
 ;;; Date: 2001-06-06
@@ -59,24 +34,8 @@
 
 (define-module (srfi srfi-1)
   :use-module (ice-9 session)
-  :use-module (ice-9 receive))
-
-(begin-deprecated
- ;; Prevent `export' from re-exporting core bindings.  This behaviour
- ;; of `export' is deprecated and will disappear in one of the next
- ;; releases.
- (define iota #f)
- (define list-copy #f)
- (define map #f)
- (define map-in-order #f)
- (define for-each #f)
- (define list-index #f)
- (define member #f)
- (define delete #f)
- (define delete! #f)
- (define assoc #f))
-
-(export
+  :use-module (ice-9 receive)
+  :export (
 ;;; Constructors
  ;; cons				<= in the core
  ;; list				<= in the core
@@ -84,9 +43,9 @@
  ;; cons*				<= in the core
  ;; make-list				<= in the core
  list-tabulate
- list-copy				; Extended.
+ list-copy
  circular-list
- iota					; Extended.
+ ;; iota				; Extended.
 
 ;;; Predicates
  proper-list?
@@ -180,20 +139,20 @@
  reduce-right
  unfold
  unfold-right
- map					; Extended.
- for-each				; Extended.
+ ;; map					; Extended.
+ ;; for-each				; Extended.
  append-map
  append-map!
  map!
- map-in-order				; Extended.
+ ;; map-in-order			; Extended.
  pair-for-each
  filter-map
 
 ;;; Filtering & partitioning
- filter
+ ;; filter				<= in the core
  partition
  remove
- filter!
+ ;; filter!				<= in the core
  partition!
  remove!
 
@@ -209,19 +168,19 @@
  break!
  any
  every
- list-index				; Extended.
- member					; Extended.
+ ;; list-index				; Extended.
+ ;; member				; Extended.
  ;; memq				<= in the core
  ;; memv				<= in the core
 
 ;;; Deletion
- delete					; Extended.
- delete!				; Extended.
+ ;; delete				; Extended.
+ ;; delete!				; Extended.
  delete-duplicates
  delete-duplicates!
 
 ;;; Association lists
- assoc					; Extended.
+ ;; assoc				; Extended.
  ;; assq				<= in the core
  ;; assv				<= in the core
  alist-cons
@@ -248,9 +207,24 @@
  ;; set-car!				<= in the core
  ;; set-cdr!				<= in the core
  )
+  :re-export (cons list cons* make-list pair? null?
+	      car cdr caar cadr cdar cddr
+	      caaar caadr cadar caddr cdaar cdadr cddar cdddr
+	      caaaar caaadr caadar caaddr cadaar cadadr caddar cadddr
+	      cdaaar cdaadr cdadar cdaddr cddaar cddadr cdddar cddddr
+	      list-ref last-pair length append append! reverse reverse!
+	      filter filter! memq memv assq assv set-car! set-cdr!)
+  :replace (iota map for-each map-in-order list-copy list-index member
+	    delete delete! assoc)
+  )
 
 (cond-expand-provide (current-module) '(srfi-1))
 
+;; Load the compiled primitives from the shared library.
+;;
+(load-extension "libguile-srfi-srfi-1-v-2" "scm_init_srfi_1")
+
+
 ;;; Constructors
 
 (define (xcons d a)
@@ -273,24 +247,10 @@
       acc
       (lp (- n 1) (cons (init-proc (- n 1)) acc)))))
 
-(define (list-copy lst)
-  (let ((ret (cons #f lst)))
-    (do ((lst lst (cdr lst))
-	 (end ret (cdr end)))
-	((not (pair? lst))
-	 (cdr ret))
-      (set-cdr! end (cons (car lst) (cdr lst))))))
-
-(define (circular-list elt1 . rest)
-  (let ((start (cons elt1 '())))
-    (let lp ((r rest) (p start))
-      (if (null? r)
-	(begin
-	  (set-cdr! p start)
-	  start)
-	(begin
-	  (set-cdr! p (cons (car r) '()))
-	  (lp (cdr r) (cdr p)))))))
+(define (circular-list elt1 . elts)
+  (set! elts (cons elt1 elts))
+  (set-cdr! (last-pair elts) elts)
+  elts)
 
 (define (iota count . rest)
   (check-arg-type non-negative-integer? count "iota")
@@ -381,16 +341,9 @@
 
 (define (car+cdr x) (values (car x) (cdr x)))
 
-(define (take x i)
-  (let lp ((n i) (l x) (acc '()))
-    (if (<= n 0)
-      (reverse! acc)
-      (lp (- n 1) (cdr l) (cons (car l) acc)))))
-(define (drop x i)
-  (let lp ((n i) (l x))
-    (if (<= n 0)
-      l
-      (lp (- n 1) (cdr l)))))
+(define take list-head)
+(define drop list-tail)
+
 (define (take-right flist i)
   (let lp ((n i) (l flist))
     (if (<= n 0)
@@ -455,45 +408,6 @@
 
 ;;; Miscelleneous: length, append, concatenate, reverse, zip & count
 
-(define (length+ clist)
-  (if (null? clist)
-    0
-    (let lp ((hare (cdr clist)) (tortoise clist) (l 1))
-      (if (null? hare)
-	l
-	(let ((hare (cdr hare)))
-	  (if (null? hare)
-	    (+ l 1)
-	    (if (eq? hare tortoise)
-	      #f
-	      (lp (cdr hare) (cdr tortoise) (+ l 2)))))))))
-
-(define (concatenate l-o-l)
-  (let lp ((l l-o-l) (acc '()))
-    (if (null? l)
-      (reverse! acc)
-      (let lp0 ((ll (car l)) (acc acc))
-	(if (null? ll)
-	  (lp (cdr l) acc)
-	  (lp0 (cdr ll) (cons (car ll) acc)))))))
-
-(define (concatenate! l-o-l)
-  (let lp0 ((l-o-l l-o-l))
-    (cond
-      ((null? l-o-l)
-       '())
-      ((null? (car l-o-l))
-       (lp0 (cdr l-o-l)))
-      (else
-       (let ((result (car l-o-l)) (tail (last-pair (car l-o-l))))
-	 (let lp ((l (cdr l-o-l)) (ntail tail))
-	   (if (null? l)
-	     result
-	     (begin
-	       (set-cdr! ntail (car l))
-	       (lp (cdr l) (last-pair ntail))))))))))
-
-
 (define (append-reverse rev-head tail)
   (let lp ((l rev-head) (acc tail))
     (if (null? l)
@@ -522,25 +436,6 @@
   (values (map1 first l) (map1 second l) (map1 third l) (map1 fourth l)
 	  (map1 fifth l)))
 
-(define (count pred clist1 . rest)
-  (if (null? rest)
-      (count1 pred clist1)
-      (let lp ((lists (cons clist1 rest)))
-	(cond ((any1 null? lists)
-	       0)
-	      (else
-	       (if (apply pred (map1 car lists))
-		 (+ 1 (lp (map1 cdr lists)))
-		 (lp (map1 cdr lists))))))))
-
-(define (count1 pred clist)
-  (let lp ((result 0) (rest clist))
-    (if (null? rest)
-	result
-	(if (pred (car rest))
-	    (lp (+ 1 result) (cdr rest))
-	    (lp result (cdr rest))))))
-
 ;;; Fold, unfold & map
 
 (define (fold kons knil list1 . rest)
@@ -627,51 +522,7 @@
 
 ;; Internal helper procedure.  Map `f' over the single list `ls'.
 ;;
-(define (map1 f ls)
-  (if (null? ls)
-      ls
-      (let ((ret (list (f (car ls)))))
-        (let lp ((ls (cdr ls)) (p ret))         ; tail pointer
-          (if (null? ls)
-              ret
-              (begin
-                (set-cdr! p (list (f (car ls))))
-                (lp (cdr ls) (cdr p))))))))
-
-;; This `map' is extended from the standard `map'.  It allows argument
-;; lists of different length, so that the shortest list determines the
-;; number of elements processed.
-;;
-(define (map f list1 . rest)
-  (if (null? rest)
-    (map1 f list1)
-    (let lp ((l (cons list1 rest)))
-      (if (any1 null? l)
-	'()
-	(cons (apply f (map1 car l)) (lp (map1 cdr l)))))))
-
-;; extended to lists of unequal length.
-(define map-in-order map)
-
-;; This `for-each' is extended from the standard `for-each'.  It
-;; allows argument lists of different length, so that the shortest
-;; list determines the number of elements processed.
-;;
-(define (for-each f list1 . rest)
-  (if (null? rest)
-    (let lp ((l list1))
-      (if (null? l)
-	(if #f #f)			; Return unspecified value.
-	(begin
-	  (f (car l))
-	  (lp (cdr l)))))
-    (let lp ((l (cons list1 rest)))
-      (if (any1 null? l)
-	(if #f #f)
-	(begin
-	  (apply f (map1 car l))
-	  (lp (map1 cdr l)))))))
-
+(define map1 map)
 
 (define (append-map f clist1 . rest)
   (if (null? rest)
@@ -696,22 +547,8 @@
 	'()
 	(append! (apply f (map1 car l)) (lp (map1 cdr l)))))))
 
-(define (map! f list1 . rest)
-  (if (null? rest)
-    (let lp ((l list1))
-      (if (null? l)
-	'()
-	(begin
-	  (set-car! l (f (car l)))
-	  (set-cdr! l (lp (cdr l)))
-	  l)))
-    (let lp ((l (cons list1 rest)) (res list1))
-      (if (any1 null? l)
-	'()
-	(begin
-	  (set-car! res (apply f (map1 car l)))
-	  (set-cdr! res (lp (map1 cdr l) (cdr res)))
-	  res)))))
+;; OPTIMIZE-ME: Re-use cons cells of list1
+(define map! map)
 
 (define (pair-for-each f clist1 . rest)
   (if (null? rest)
@@ -747,33 +584,9 @@
 
 ;;; Filtering & partitioning
 
-(define (filter pred list)
-  (check-arg-type list? list "filter")  ; reject circular lists.
-  (letrec ((filiter (lambda (pred rest result)
-		      (if (null? rest)
-			  (reverse! result)
-			  (filiter pred (cdr rest)
-				   (cond ((pred (car rest))
-					  (cons (car rest) result))
-					 (else
-					  result)))))))
-    (filiter pred list '())))
-
-(define (partition pred list)
-  (if (null? list)
-    (values '() '())
-    (if (pred (car list))
-      (receive (in out) (partition pred (cdr list))
-	       (values (cons (car list) in) out))
-      (receive (in out) (partition pred (cdr list))
-	       (values in (cons (car list) out))))))
-
 (define (remove pred list)
   (filter (lambda (x) (not (pred x))) list))
 
-(define (filter! pred list)
-  (filter pred list))			; XXX:optimize
-
 (define (partition! pred list)
   (partition pred list))		; XXX:optimize
 
@@ -895,66 +708,8 @@
 	    (else
 	     (lp (map1 cdr lists) (+ i 1)))))))
 
-(define (member x list . rest)
-  (let ((l= (if (pair? rest) (car rest) equal?)))
-    (let lp ((l list))
-      (if (null? l)
-	#f
-	(if (l= x (car l))
-	  l
-	  (lp (cdr l)))))))
-
-;;; Deletion
-
-(define (delete x list . rest)
-  (let ((l= (if (pair? rest) (car rest) equal?)))
-    (let lp ((l list))
-      (if (null? l)
-	'()
-	(if (l= x (car l))
-	  (lp (cdr l))
-	  (cons (car l) (lp (cdr l))))))))
-
-(define (delete! x list . rest)
-  (let ((l= (if (pair? rest) (car rest) equal?)))
-    (delete x list l=)))		; XXX:optimize
-
-(define (delete-duplicates list . rest)
-  (let ((l= (if (pair? rest) (car rest) equal?)))
-    (let lp0 ((l1 list))
-      (if (null? l1)
-	'()
-	(if (let lp1 ((l2 (cdr l1)))
-	      (if (null? l2)
-		#f
-		(if (l= (car l1) (car l2))
-		  #t
-		  (lp1 (cdr l2)))))
-	  (lp0 (cdr l1))
-	  (cons (car l1) (lp0 (cdr l1))))))))
-
-(define (delete-duplicates list . rest)
-  (let ((l= (if (pair? rest) (car rest) equal?)))
-    (let lp ((list list))
-      (if (null? list)
-	'()
-	(cons (car list) (lp (delete (car list) (cdr list) l=)))))))
-
-(define (delete-duplicates! list . rest)
-  (let ((l= (if (pair? rest) (car rest) equal?)))
-    (delete-duplicates list l=)))	; XXX:optimize
-
 ;;; Association lists
 
-(define (assoc key alist . rest)
-  (let ((k= (if (pair? rest) (car rest) equal?)))
-    (let lp ((a alist))
-      (if (null? a)
-	#f
-	(if (k= key (caar a))
-	  (car a)
-	  (lp (cdr a)))))))
-
 (define (alist-cons key datum alist)
   (acons key datum alist))
 
